@@ -11,6 +11,9 @@ import nox
 if TYPE_CHECKING:
     from collections.abc import Sequence
 
+nox.needs_version = ">=2024.3.2"
+nox.options.default_venv_backend = "uv|virtualenv"
+
 DIR = Path(__file__).parent.resolve()
 
 nox.options.sessions = [
@@ -54,12 +57,11 @@ def _run_tests(
     posargs = list(session.posargs)
     env = {"PIP_DISABLE_PIP_VERSION_CHECK": "1"}
 
-    _install_args = list(install_args)
     # This will not work if system CMake is too old (<3.15)
     if shutil.which("cmake") is None and shutil.which("cmake3") is None:
-        _install_args.append("cmake")
+        session.install("cmake")
     if shutil.which("ninja") is None:
-        _install_args.append("ninja")
+        session.install("ninja")
 
     _extras = ["test", *extras]
     if "--cov" in posargs:
@@ -68,6 +70,7 @@ def _run_tests(
 
     install_arg = f"-e.[{','.join(_extras)}]"
     session.install(install_arg, *install_args)
+    session.run("pip", "list")
     session.run("pytest", *run_args, *posargs, env=env)
 
 
@@ -90,7 +93,7 @@ def tests(session: nox.Session) -> None:
     """
     Run the unit and regular tests. Includes coverage if --cov passed.
     """
-    _run_tests(session, extras=["test-meta,test-numpy,test-schema"])
+    _run_tests(session, extras=["test-meta,test-numpy,test-schema,test-hatchling"])
 
 
 @nox.session(reuse_venv=True)
@@ -120,31 +123,21 @@ def minimums(session: nox.Session) -> None:
 @nox.session(reuse_venv=True)
 def docs(session: nox.Session) -> None:
     """
-    Build the docs. Pass "--serve" to serve. Pass "-b linkcheck" to check links.
+    Build the docs. Use "--non-interactive" to avoid serving. Pass "-b linkcheck" to check links.
     """
 
     parser = argparse.ArgumentParser()
-    parser.add_argument("--serve", action="store_true", help="Serve after building")
     parser.add_argument(
         "-b", dest="builder", default="html", help="Build target (default: html)"
     )
     args, posargs = parser.parse_known_args(session.posargs)
 
-    if args.builder != "html" and args.serve:
-        session.error("Must not specify non-HTML builder with --serve")
+    serve = args.builder == "html" and session.interactive
+    extra_installs = ["sphinx-autobuild"] if serve else []
+    session.install("-e.[docs]", *extra_installs)
 
-    extra_installs = ["sphinx-autobuild"] if args.serve else []
-
-    session.install("-e.[docs,pyproject]", *extra_installs)
-    session.run("pip", "list")
     session.chdir("docs")
 
-    if args.builder == "linkcheck":
-        session.run(
-            "sphinx-build", "-b", "linkcheck", ".", "_build/linkcheck", *posargs
-        )
-        return
-
     shared_args = (
         "-n",  # nitpicky mode
         "-T",  # full tracebacks
@@ -154,8 +147,8 @@ def docs(session: nox.Session) -> None:
         *posargs,
     )
 
-    if args.serve:
-        session.run("sphinx-autobuild", *shared_args)
+    if serve:
+        session.run("sphinx-autobuild", "--open-browser", *shared_args)
     else:
         session.run("sphinx-build", "--keep-going", *shared_args)
 
@@ -236,7 +229,7 @@ def downstream(session: nox.Session) -> None:
     proj_dir = tmp_dir / "_".join(args.project.split("/"))
 
     session.install("build", "hatch-vcs", "hatchling")
-    session.install(".[pyproject]", "--no-build-isolation")
+    session.install(".", "--no-build-isolation")
 
     if proj_dir.is_dir():
         session.chdir(proj_dir)
@@ -289,3 +282,29 @@ def downstream(session: nox.Session) -> None:
 
     if args.code:
         session.run("python", "-c", args.code)
+
+
+@nox.session(venv_backend="none")
+def vendor_pyproject_metadata(session: nox.Session) -> None:
+    """
+    Vendor pyproject.toml metadata.
+    """
+
+    parser = argparse.ArgumentParser(
+        prog=f"{Path(sys.argv[0]).name} -s vendor_pyproject_metadata"
+    )
+    parser.add_argument("version", help="A tag or ref to vendor")
+    args = parser.parse_args(session.posargs)
+
+    session.run(
+        "curl",
+        "-o",
+        "src/scikit_build_core/_vendor/pyproject_metadata/__init__.py",
+        f"https://raw.githubusercontent.com/pypa/pyproject-metadata/{args.version}/pyproject_metadata/__init__.py",
+    )
+    session.run(
+        "curl",
+        "-o",
+        "src/scikit_build_core/_vendor/pyproject_metadata/LICENSE",
+        f"https://raw.githubusercontent.com/pypa/pyproject-metadata/{args.version}/LICENSE",
+    )
